package com.loafle.overflow.central.module.member.service;

import com.loafle.overflow.central.module.member.dao.MemberTotpDAO;
import com.loafle.overflow.core.exception.OverflowException;
import com.loafle.overflow.model.member.Member;
import com.loafle.overflow.model.member.MemberTotp;
import com.loafle.overflow.service.central.member.MemberTotpService;
import com.warrenstrange.googleauth.GoogleAuthenticator;
import com.warrenstrange.googleauth.GoogleAuthenticatorKey;
import org.apache.http.client.utils.URIBuilder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Map;

/**
 * Created by geek on 18. 3. 8.
 */
@Service("MemberTotpService")
public class CentralMemberTotpService implements MemberTotpService {
  @Autowired
  private MemberTotpDAO totpDAO;

  public void regist(Member member, String secretCode, String code) throws OverflowException {

    if (null == member || 0 >= member.getId()) {
      throw new OverflowException("SignInIdNotExistException", new Throwable());
    }

    if (!this.checkCode(secretCode, code)) {
      throw new OverflowException("TotpCodeNotMatchException", new Throwable());
    }

    MemberTotp totp = new MemberTotp();
    totp.setMember(member);
    totp.setSecretCode(secretCode);

    this.totpDAO.save(totp);
  }

  public MemberTotp modify(MemberTotp totp) throws OverflowException {
    if (null == totp.getSecretCode() || totp.getSecretCode().equals("")) {
      throw new OverflowException("SecretCodeNotExistException", new Throwable());
    }
    if (null == totp.getMember() || 0 < totp.getMember().getId()) {
      throw new OverflowException("SignInIdNotExistException", new Throwable());
    }

    return this.totpDAO.save(totp);
  }

  public void remove(Long id) throws OverflowException {
    this.totpDAO.deleteById(id);
  }

  public MemberTotp read(Long id) throws OverflowException {
    return this.totpDAO.findById(id).get();
  }

  public boolean checkCodeForMember(String memberEmail, String code) throws OverflowException {
    MemberTotp totp = this.totpDAO.findByMemberEmail(memberEmail);
    if (null == totp || totp.getSecretCode().equals("")) {
      throw new OverflowException("SignInIdNotExistException", new Throwable());
    }
    return this.checkCode(totp.getSecretCode(), code);
  }

  public boolean checkCode(String secretCode, String code) throws OverflowException {
    GoogleAuthenticator googleAuthenticator = new GoogleAuthenticator();
    int codeInt = Integer.parseInt(code);
    boolean isCheck = googleAuthenticator.authorize(secretCode, codeInt);

    if (!isCheck) {
      throw new OverflowException("TotpCodeNotMatchException", new Throwable());
    }

    return isCheck;
  }

  public Map<String, String> createTotp(Member member) throws OverflowException {
    Map<String, String> returnMap = new HashMap<>();

    GoogleAuthenticator googleAuthenticator = new GoogleAuthenticator();
    final GoogleAuthenticatorKey key = googleAuthenticator.createCredentials();

    String secret = key.getKey();
    // List<Integer> scratchCodes = key.getScratchCodes();
    // String otpAuthURL = GoogleAuthenticatorQRGenerator.getOtpAuthURL("overFlow",
    // member.getEmail(), key);
    URIBuilder uri = (new URIBuilder()).setScheme("otpauth").setHost("totp")
        .setPath("/" + formatLabel("overFlow", member.getEmail())).setParameter("secret", key.getKey());

    returnMap.put("key", secret);
    returnMap.put("uri", uri.toString());

    return returnMap;
  }

  private String formatLabel(String issuer, String accountName) throws OverflowException {
    if (accountName != null && accountName.trim().length() != 0) {
      StringBuilder sb = new StringBuilder();
      if (issuer != null) {
        if (issuer.contains(":")) {
          throw new IllegalArgumentException("Issuer cannot contain the ':' character.");
        }

        sb.append(issuer);
        sb.append(":");
      }

      sb.append(accountName);
      return sb.toString();
    } else {
      throw new IllegalArgumentException("Account name must not be empty.");
    }
  }
}
