package com.loafle.overflow.central.module.target.service;

import java.util.ArrayList;
import java.util.List;

import com.loafle.overflow.central.commons.utils.PageUtil;
import com.loafle.overflow.central.module.probe.service.CentralProbeService;
import com.loafle.overflow.central.module.target.dao.TargetDAO;
import com.loafle.overflow.core.exception.OverflowException;
import com.loafle.overflow.core.model.PageParams;
import com.loafle.overflow.core.type.CryptoType;
import com.loafle.overflow.model.discovery.Host;
import com.loafle.overflow.model.discovery.Service;
import com.loafle.overflow.model.infra.Infra;
import com.loafle.overflow.model.infra.InfraHost;
import com.loafle.overflow.model.meta.MetaInfraType;
import com.loafle.overflow.model.probe.Probe;
import com.loafle.overflow.model.target.Target;
import com.loafle.overflow.service.central.infra.InfraService;
import com.loafle.overflow.service.central.target.TargetService;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.transaction.annotation.Transactional;

/**
 * Created by insanity on 17. 6. 28.
 */
@org.springframework.stereotype.Service("TargetService")
public class CentralTargetService implements TargetService {

  @Autowired
  private TargetDAO targetDAO;
  @Autowired
  private CentralProbeService probeService;
  // @Autowired
  // private CentralInfraHostService infraHostService;
  // @Autowired
  // private CentralInfraServiceService infraServiceService;
  @Autowired
  private InfraService infraService;

  @Transactional
  public Target regist(Target target, Long probeID) throws OverflowException {
    this.probeService.increaseTargetCount(probeID);
    return this.targetDAO.save(target);
  }

  @Transactional
  public void remove(Long id, Long probeID) throws OverflowException {
    this.probeService.decreaseTargetCount(probeID);
    this.targetDAO.deleteById(id);
  }

  public Target read(Long id) throws OverflowException {
    return this.targetDAO.findById(id).get();
  }

  public Target modify(Target target) throws OverflowException {
    return this.targetDAO.save(target);
  }

  public Target increaseSensorCount(Long id) throws OverflowException {
    Target t = this.targetDAO.findById(id).get();
    t.setSensorCount(t.getSensorCount() + 1);
    return this.targetDAO.save(t);
  }

  public Target decreaseSensorCount(Long id) throws OverflowException {
    Target t = this.targetDAO.findById(id).get();
    int count = t.getSensorCount();
    if (t.getSensorCount() > 0) {
      t.setSensorCount(count - 1);
    }
    return this.targetDAO.save(t);
  }

  public Target readExistHostTarget(Long probeId, String ip) throws OverflowException {
    // InfraHost infraHost = this.infraHostService.readByProbeIdAndIpv4(probeId,
    // ip);
    // if (null == infraHost)
    // return null;
    // return this.targetDAO.findByInfraId(infraHost.getId());
    return null;
  }

  public Target readExistServiceTarget(Long hostId, int portNumber, String portType) throws OverflowException {
    // InfraService infraService =
    // this.infraServiceService.readByInfraHostIDAndPortAndPortType(hostId,
    // portNumber,
    // portType);
    // if (null == infraService)
    // return null;
    // return this.targetDAO.findByInfraId(infraService.getId());
    return null;
  }

  public Page<Target> readAllByProbeID(Long probeID, PageParams pageParams) throws OverflowException {
    return this.targetDAO.findAllByInfraProbeId(probeID, PageUtil.getPageRequest(pageParams));
  }

  @Transactional
  public List<Target> registDiscoveredTargets(Long probeID, List<Host> hosts, List<Service> services)
      throws OverflowException {
    List<Target> targets = new ArrayList<Target>();
    for (Host host : hosts) {
      InfraHost infraHost = this.infraService.regist(probeID, host);
      String displayName = (host.getIpv6() == null || host.getIpv6().trim().isEmpty()) ? host.getIpv6()
          : host.getIpv4();
      Target target = new Target();
      target.setInfra(infraHost);
      target.setSensorCount(0);
      target.setName(displayName);
      targets.add(target);
    }
    for (Service service : services) {
      com.loafle.overflow.model.infra.InfraService infraService = this.infraService.regist(probeID, service);
      Target target = new Target();
      target.setInfra(infraService);
      target.setSensorCount(0);
      target.setName(infraService.getMetaInfraType().getName());
      targets.add(target);
    }

    return this.targetDAO.saveAll(targets);
  }

  // private List<Target> registDiscoveredHostTargets(Long probeId, List<Host>
  // hosts) throws OverflowException {
  // List<Target> targets = new ArrayList<>();
  // if (null == hosts)
  // return targets;
  // for (Host host : hosts) {
  // InfraHost infraHost = this.registInfraHostByDiscoveredHost(host, probeId);
  // Target target = new Target();
  // target.setInfra(infraHost);
  // String displayName = (host.getIpv6() == null ||
  // host.getIpv6().trim().isEmpty()) ? host.getIpv6()
  // : host.getIpv4();
  // target.setDisplayName(displayName);
  // targets.add(this.targetDAO.save(target));
  // }
  // return targets;
  // return null;
  // }

  // private List<Target> registDiscoveredServiceTargets(Long probeId,
  // List<Service> services) throws OverflowException {
  // List<Target> targets = new ArrayList<>();
  // if (null == services)
  // return targets;
  // for (Service service : services) {
  // Host host = service.getPort().getHost();
  // InfraHost infraHost = null;
  // InfraHost existInfraHost =
  // this.infraHostService.readByProbeIdAndIpv4(probeId, host.getIpv4());
  // InfraService infraService = null;
  // if (null != existInfraHost) {
  // infraHost = existInfraHost;
  // } else {
  // infraHost = this.registInfraHostByDiscoveredHost(host, probeId);
  // infraService = this.registInfraServiceByDiscoveredService(infraHost, service,
  // probeId);
  // }

  // Target target = new Target();
  // target.setInfra(infraService);
  // String displayName = service.getServiceName();
  // target.setDisplayName(displayName);
  // targets.add(this.targetDAO.save(target));
  // }
  // return targets;
  // }

  // private InfraHost registInfraHostByDiscoveredHost(Host host, Long probeId)
  // throws OverflowException {
  // InfraHost infraHost = new InfraHost();
  // infraHost.setMetaInfraType(new MetaInfraType(2));
  // infraHost.setProbe(new Probe(probeId));
  // // infraHost.setIpv4(host.getIpv4());
  // // infraHost.setIpv6(host.getIpv6());
  // // infraHost.setMac(host.getMac());
  // return this.infraHostService.regist(infraHost);
  // }

  // private InfraService registInfraServiceByDiscoveredService(InfraHost
  // infraHost, Service service, Long probeId)
  // throws OverflowException {
  // InfraService infraService = new InfraService();
  // infraHost.setMetaInfraType(new MetaInfraType(7));
  // // infraService.setInfraHost(infraHost);
  // // infraService.setPort(service.getPort().getPortNumber());
  // // infraService.setPortType(service.getPort().getPortType().toString());
  // // infraService.setTlsType(service.getCryptoType() == CryptoType.TLS);
  // //
  // infraService.setMetaInfraVendor(MetaInfraVendor.CreateInfraVendorByService(service.getServiceName()));
  // return this.infraServiceService.regist(infraService);
  // }

}
